@@ -36,6 +36,7 @@
 #include <unistd.h>
 
 #include <glib/gi18n.h>
+#include <libpeas/peas-extension-set.h>
 
 #include "gedit-app.h"
 #include "gedit-commands.h"
@@ -45,6 +46,8 @@
 #include "gedit-enum-types.h"
 #include "gedit-dirs.h"
 #include "gedit-settings.h"
+#include "gedit-app-activatable.h"
+#include "gedit-plugins-engine.h"
 
 #ifdef OS_OSX
 #include "gedit-app-osx.h"
@@ -78,8 +81,10 @@ struct _GeditAppPrivate
 	GtkPageSetup      *page_setup;
 	GtkPrintSettings  *print_settings;
 	
-	GSettings         *settings;
+	GObject           *settings;
 	GSettings         *window_settings;
+
+	PeasExtensionSet  *extensions;
 };
 
 G_DEFINE_ABSTRACT_TYPE(GeditApp, gedit_app, G_TYPE_INITIALLY_UNOWNED)
@@ -102,7 +107,7 @@ gedit_app_finalize (GObject *object)
 static void
 gedit_app_dispose (GObject *object)
 {
-	GeditApp *app = GEDIT_APP (object); 
+	GeditApp *app = GEDIT_APP (object);
 
 	if (app->priv->window_settings != NULL)
 	{
@@ -116,6 +121,16 @@ gedit_app_dispose (GObject *object)
 		app->priv->settings = NULL;
 	}
 
+	if (app->priv->extensions != NULL)
+	{
+		peas_extension_set_call (app->priv->extensions,
+					 "deactivate",
+					 app);
+
+		g_object_unref (app->priv->extensions);
+		app->priv->extensions = NULL;
+	}
+
 	G_OBJECT_CLASS (gedit_app_parent_class)->dispose (object);
 }
 
@@ -300,26 +315,32 @@ ensure_user_config_dir (void)
 static void
 load_accels (void)
 {
-	const gchar *filename;
+	gchar *filename;
 
-	filename = gedit_dirs_get_user_accels_file ();
+	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
+				     "accels",
+				     NULL);
 	if (filename != NULL)
 	{
 		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
 		gtk_accel_map_load (filename);
+		g_free (filename);
 	}
 }
 
 static void
 save_accels (void)
 {
-	const gchar *filename;
+	gchar *filename;
 
-	filename = gedit_dirs_get_user_accels_file ();
+	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
+				     "accels",
+				     NULL);
 	if (filename != NULL)
 	{
 		gedit_debug_message (DEBUG_APP, "Saving keybindings in %s\n", filename);
 		gtk_accel_map_save (filename);
+		g_free (filename);
 	}
 }
 
@@ -467,19 +488,54 @@ save_print_settings (GeditApp *app)
 	g_free (filename);
 }
 
+static void
+extension_added (PeasExtensionSet *extensions,
+		 PeasPluginInfo   *info,
+		 PeasExtension    *exten,
+		 GeditApp         *app)
+{
+	peas_extension_call (exten, "activate");
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+		   PeasPluginInfo   *info,
+		   PeasExtension    *exten,
+		   GeditApp         *app)
+{
+	peas_extension_call (exten, "deactivate");
+}
+
 static void
 gedit_app_init (GeditApp *app)
 {
 	app->priv = GEDIT_APP_GET_PRIVATE (app);
 
 	load_accels ();
-	
+
 	/* Load settings */
 	app->priv->settings = gedit_settings_new ();
 	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
 
 	/* initial lockdown state */
 	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
+
+	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gedit_plugins_engine_get_default ()),
+							GEDIT_TYPE_APP_ACTIVATABLE,
+							"app", app,
+							NULL);
+
+	g_signal_connect (app->priv->extensions,
+			  "extension-added",
+			  G_CALLBACK (extension_added),
+			  app);
+
+	g_signal_connect (app->priv->extensions,
+			  "extension-removed",
+			  G_CALLBACK (extension_removed),
+			  app);
+
+	peas_extension_set_call (app->priv->extensions, "activate");
 }
 
 /**
@@ -488,7 +544,7 @@ gedit_app_init (GeditApp *app)
  * Returns the #GeditApp object. This object is a singleton and
  * represents the running gedit instance.
  *
- * Return value: the #GeditApp pointer
+ * Return value: (transfer none): the #GeditApp pointer
  */
 GeditApp *
 gedit_app_get_default (void)
@@ -709,7 +765,7 @@ gedit_app_create_window_real (GeditApp    *app,
  *
  * Create a new #GeditWindow part of @app.
  *
- * Return value: the new #GeditWindow
+ * Return value: (transfer none): the new #GeditWindow
  */
 GeditWindow *
 gedit_app_create_window (GeditApp  *app,
@@ -746,7 +802,7 @@ _gedit_app_restore_window (GeditApp    *app,
  *
  * Returns all the windows currently present in #GeditApp.
  *
- * Return value: (element-type Window) (transfer none): the list of #GeditWindows objects.
+ * Return value: (element-type Gedit.Window) (transfer none): the list of #GeditWindows objects.
  * The list should not be freed
  */
 const GList *
@@ -763,7 +819,7 @@ gedit_app_get_windows (GeditApp *app)
  *
  * Retrives the #GeditWindow currently active.
  *
- * Return value: the active #GeditWindow
+ * Return value: (transfer none): the active #GeditWindow
  */
 GeditWindow *
 gedit_app_get_active_window (GeditApp *app)
@@ -821,7 +877,8 @@ is_in_viewport (GeditWindow  *window,
 	/* Check for viewport match */
 	gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
 	gdk_window_get_position (gdkwindow, &x, &y);
-	gdk_drawable_get_size (gdkwindow, &width, &height);
+	width = gdk_window_get_width (gdkwindow);
+	height = gdk_window_get_height (gdkwindow);
 	gedit_utils_get_current_viewport (screen, &vp_x, &vp_y);
 	x += vp_x;
 	y += vp_y;
@@ -1067,7 +1124,7 @@ _gedit_app_set_default_print_settings (GeditApp         *app,
 	app->priv->print_settings = g_object_ref (settings);
 }
 
-GSettings *
+GObject *
 _gedit_app_get_settings (GeditApp *app)
 {
 	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
