@@ -67,16 +67,13 @@ G_DEFINE_TYPE (GeditCommandLine, gedit_command_line, G_TYPE_INITIALLY_UNOWNED)
 static void
 gedit_command_line_finalize (GObject *object)
 {
-	GeditCommandLine *command_line = GEDIT_COMMAND_LINE (object);
+	GeditCommandLinePrivate *priv = GEDIT_COMMAND_LINE (object)->priv;
 
-	g_free (command_line->priv->encoding_charset);
-	g_free (command_line->priv->line_column_position);
-	g_strfreev (command_line->priv->remaining_args);
-
-	g_free (command_line->priv->geometry);
-
-	g_slist_foreach (command_line->priv->file_list, (GFunc)g_object_unref, NULL);
-	g_slist_free (command_line->priv->file_list);
+	g_free (priv->encoding_charset);
+	g_free (priv->line_column_position);
+	g_strfreev (priv->remaining_args);
+	g_free (priv->geometry);
+	g_slist_free_full (priv->file_list, g_object_unref);
 
 	G_OBJECT_CLASS (gedit_command_line_parent_class)->finalize (object);
 }
@@ -335,15 +332,13 @@ gedit_command_line_parse (GeditCommandLine   *command_line,
 	/* Setup command line options */
 	context = g_option_context_new (_("- Edit text files"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (FALSE));
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
 
 #ifdef ENABLE_INTROSPECTION
 	g_option_context_add_group (context, g_irepository_get_option_group ());
 #endif
 
-	gtk_init (argc, argv);
-
 	if (!g_option_context_parse (context, argc, argv, &error))
 	{
 		g_print(_("%s\nRun '%s --help' to see a full list of available command line options.\n"),
