@@ -46,47 +46,30 @@
 #include "gedit-notebook.h"
 #include "gedit-tab.h"
 #include "gedit-tab-label.h"
-#include "gedit-marshal.h"
 #include "gedit-window.h"
 #include "gedit-enum-types.h"
 #include "gedit-settings.h"
 
-#define AFTER_ALL_TABS -1
-#define NOT_IN_APP_WINDOWS -2
-
 #define GEDIT_NOTEBOOK_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_NOTEBOOK, GeditNotebookPrivate))
 
+#define GEDIT_NOTEBOOK_GROUP_NAME "GeditNotebookGroup"
+
 struct _GeditNotebookPrivate
 {
 	GSettings     *ui_settings;
 
 	GList         *focused_pages;
-	gulong         motion_notify_handler_id;
-
-	gint           x_start;
-	gint           y_start;
 
 	GeditNotebookShowTabsModeType show_tabs_mode;
 
-	guint          drag_in_progress : 1;
+	gint           xthickness;
+
 	guint          close_buttons_sensitive : 1;
-	guint          tab_drag_and_drop_enabled : 1;
-	guint          destroy_has_run : 1;
+	guint          collapse_border : 1;
 };
 
 G_DEFINE_TYPE(GeditNotebook, gedit_notebook, GTK_TYPE_NOTEBOOK)
 
-static gboolean gedit_notebook_change_current_page (GtkNotebook   *notebook,
-						    gint           offset);
-
-static void move_current_tab_to_another_notebook  (GeditNotebook  *src,
-						   GeditNotebook  *dest,
-						   GdkEventMotion *event,
-						   gint            dest_position);
-
-/* Local variables */
-static GdkCursor *cursor = NULL;
-
 /* Properties */
 enum
 {
@@ -97,15 +80,12 @@ enum
 /* Signals */
 enum
 {
-	TABS_REORDERED,
-	TAB_DETACHED,
 	TAB_CLOSE_REQUEST,
 	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-
 static void
 update_tabs_visibility (GeditNotebook *notebook,
 			gboolean       before_inserting)
@@ -215,6 +195,207 @@ gedit_notebook_grab_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (tab);
 }
 
+static void
+gedit_notebook_style_set (GtkWidget *widget,
+			  GtkStyle  *previous_style)
+{
+	GeditNotebook *nb = GEDIT_NOTEBOOK (widget);
+
+	if (previous_style != NULL && nb->priv->collapse_border)
+	{
+		previous_style->xthickness = 0;
+	}
+
+	GTK_WIDGET_CLASS (gedit_notebook_parent_class)->style_set (widget, previous_style);
+}
+
+/*
+ * We need to override this because when we don't show the tabs, like in
+ * fullscreen we need to have wrap around too
+ */
+static gboolean
+gedit_notebook_change_current_page (GtkNotebook *notebook,
+				    gint         offset)
+{
+	gboolean wrap_around;
+	gint current;
+
+	current = gtk_notebook_get_current_page (notebook);
+
+	if (current != -1)
+	{
+		current = current + offset;
+
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
+			      "gtk-keynav-wrap-around", &wrap_around,
+			      NULL);
+
+		if (wrap_around)
+		{
+			if (current < 0)
+			{
+				current = gtk_notebook_get_n_pages (notebook) - 1;
+			}
+			else if (current >= gtk_notebook_get_n_pages (notebook))
+			{
+				current = 0;
+			}
+		}
+
+		gtk_notebook_set_current_page (notebook, current);
+	}
+	else
+	{
+		gtk_widget_error_bell (GTK_WIDGET (notebook));
+	}
+
+	return TRUE;
+}
+
+static void
+gedit_notebook_switch_page (GtkNotebook *notebook,
+                            GtkWidget   *page,
+                            guint        page_num)
+{
+	GeditNotebook *nb = GEDIT_NOTEBOOK (notebook);
+
+	GTK_NOTEBOOK_CLASS (gedit_notebook_parent_class)->switch_page (notebook, page, page_num);
+
+	/* Remove the old page, we dont want to grow unnecessarily
+	 * the list */
+	if (nb->priv->focused_pages)
+	{
+		nb->priv->focused_pages =
+			g_list_remove (nb->priv->focused_pages, page);
+	}
+
+	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
+	                                         page);
+
+	/* give focus to the tab */
+	gtk_widget_grab_focus (page);
+}
+
+static void
+on_tab_label_destroyed (GtkWidget *tab_label,
+                        GeditTab  *tab)
+{
+	g_object_set_data (G_OBJECT (tab), "tab-label", NULL);
+}
+
+static void
+close_button_clicked_cb (GeditTabLabel *tab_label, GeditNotebook *notebook)
+{
+	GeditTab *tab;
+
+	tab = gedit_tab_label_get_tab (tab_label);
+	g_signal_emit (notebook, signals[TAB_CLOSE_REQUEST], 0, tab);
+}
+
+static void
+smart_tab_switching_on_closure (GeditNotebook *nb,
+				GeditTab      *tab)
+{
+	gboolean jump_to;
+
+	jump_to = GPOINTER_TO_INT (g_object_get_data
+	                           (G_OBJECT (tab), "jump_to"));
+
+	if (!jump_to || !nb->priv->focused_pages)
+	{
+		gtk_notebook_next_page (GTK_NOTEBOOK (nb));
+	}
+	else
+	{
+		GList *l;
+		GtkWidget *child;
+		int page_num;
+
+		/* activate the last focused tab */
+		l = g_list_last (nb->priv->focused_pages);
+		child = GTK_WIDGET (l->data);
+		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
+		                                  child);
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
+		                               page_num);
+	}
+}
+
+static GtkWidget *
+get_tab_label (GeditTab *tab)
+{
+	GObject *tab_label;
+
+	tab_label = g_object_get_data (G_OBJECT (tab), "tab-label");
+
+	return (tab_label != NULL) ? GTK_WIDGET (tab_label) : NULL;
+}
+
+static void
+gedit_notebook_page_removed (GtkNotebook *notebook,
+                             GtkWidget   *page,
+                             guint        page_num)
+{
+	GeditNotebook *nb = GEDIT_NOTEBOOK (notebook);
+	gint num_pages;
+	gint curr;
+	GtkWidget *tab_label;
+
+	tab_label = get_tab_label (GEDIT_TAB (page));
+
+	if (tab_label != NULL)
+	{
+		g_signal_handlers_disconnect_by_func (tab_label,
+		                                      G_CALLBACK (on_tab_label_destroyed),
+		                                      page);
+		g_signal_handlers_disconnect_by_func (tab_label,
+		                                      G_CALLBACK (close_button_clicked_cb),
+		                                      nb);
+	}
+
+	/* Remove the page from the focused pages list */
+	nb->priv->focused_pages =  g_list_remove (nb->priv->focused_pages,
+	                                          page);
+
+	curr = gtk_notebook_get_current_page (notebook);
+
+	if (page_num == curr)
+	{
+		smart_tab_switching_on_closure (nb, GEDIT_TAB (page));
+	}
+
+	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
+
+	/* If there is no tabs, calling this is pointless */
+	if (num_pages > 0)
+	{
+		update_tabs_visibility (nb, FALSE);
+	}
+}
+
+static void
+gedit_notebook_page_added (GtkNotebook *notebook,
+                           GtkWidget   *page,
+                           guint        page_num)
+{
+	GeditNotebook *nb = GEDIT_NOTEBOOK (notebook);
+	GtkWidget *tab_label;
+
+	tab_label = get_tab_label (GEDIT_TAB (page));
+
+	g_signal_connect (tab_label,
+	                  "destroy",
+	                  G_CALLBACK (on_tab_label_destroyed),
+	                  page);
+
+	g_signal_connect (tab_label,
+	                  "close-clicked",
+	                  G_CALLBACK (close_button_clicked_cb),
+	                  nb);
+
+	update_tabs_visibility (GEDIT_NOTEBOOK (notebook), FALSE);
+}
+
 static void
 gedit_notebook_class_init (GeditNotebookClass *klass)
 {
@@ -228,8 +409,12 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 	object_class->set_property = gedit_notebook_set_property;
 
 	gtkwidget_class->grab_focus = gedit_notebook_grab_focus;
-	
+	gtkwidget_class->style_set = gedit_notebook_style_set;
+
 	notebook_class->change_current_page = gedit_notebook_change_current_page;
+	notebook_class->switch_page = gedit_notebook_switch_page;
+	notebook_class->page_removed = gedit_notebook_page_removed;
+	notebook_class->page_added = gedit_notebook_page_added;
 
 	g_object_class_install_property (object_class, PROP_SHOW_TABS_MODE,
 					 g_param_spec_enum ("show-tabs-mode",
@@ -239,25 +424,6 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 							    GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS,
 							    G_PARAM_READWRITE));
 
-	signals[TAB_DETACHED] =
-		g_signal_new ("tab_detached",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GeditNotebookClass, tab_detached),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      GEDIT_TYPE_TAB);
-	signals[TABS_REORDERED] =
-		g_signal_new ("tabs_reordered",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GeditNotebookClass, tabs_reordered),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
 	signals[TAB_CLOSE_REQUEST] =
 		g_signal_new ("tab-close-request",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -272,468 +438,6 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 	g_type_class_add_private (object_class, sizeof (GeditNotebookPrivate));
 }
 
-static GeditNotebook *
-get_notebook_from_widget (GtkWidget *widget)
-{
-	if (widget == NULL)
-	{
-		return NULL;
-	}
-
-	while (!gtk_widget_is_toplevel (GTK_WIDGET (widget)))
-	{
-		if (GEDIT_IS_NOTEBOOK (widget))
-		{
-			return GEDIT_NOTEBOOK (widget);
-		}
-
-		widget = gtk_widget_get_parent (GTK_WIDGET (widget));
-	}
-
-	return NULL;
-}
-
-static GeditNotebook *
-find_notebook_at_pointer (gint abs_x,
-			  gint abs_y)
-{
-	GdkWindow *win_at_pointer;
-	gpointer widget = NULL;
-	gint x, y;
-
-	/* FIXME multi-head */
-	win_at_pointer = gdk_window_at_pointer (&x, &y);
-	if (win_at_pointer == NULL)
-	{
-		/* We are outside all windows of the same application */
-		return NULL;
-	}
-
-	/* get the GtkWidget which owns the toplevel GdkWindow */
-	gdk_window_get_user_data (win_at_pointer, &widget);
-
-	return get_notebook_from_widget (widget);
-}
-
-static gboolean
-is_in_notebook_window (GeditNotebook *notebook,
-		       gint           abs_x, 
-		       gint           abs_y)
-{
-	GeditNotebook *nb_at_pointer;
-
-	g_return_val_if_fail (notebook != NULL, FALSE);
-
-	nb_at_pointer = find_notebook_at_pointer (abs_x, abs_y);
-
-	return (nb_at_pointer == notebook);
-}
-
-static gint
-find_tab_num_at_pos (GeditNotebook *notebook, 
-		     gint           abs_x, 
-		     gint           abs_y)
-{
-	GtkPositionType tab_pos;
-	int page_num = 0;
-	GtkNotebook *nb = GTK_NOTEBOOK (notebook);
-	GtkWidget *page;
-
-	tab_pos = gtk_notebook_get_tab_pos (GTK_NOTEBOOK (notebook));
-
-	/* For some reason unfullscreen + quick click can
-	   cause a wrong click event to be reported to the tab */
-	if (!is_in_notebook_window (notebook, abs_x, abs_y))
-	{
-		return NOT_IN_APP_WINDOWS;
-	}
-
-	while ((page = gtk_notebook_get_nth_page (nb, page_num)) != NULL)
-	{
-		GtkWidget *tab;
-		gint max_x, max_y;
-		gint x_root, y_root;
-		GtkAllocation allocation;
-
-		tab = gtk_notebook_get_tab_label (nb, page);
-		g_return_val_if_fail (tab != NULL, AFTER_ALL_TABS);
-
-		if (!gtk_widget_get_mapped (tab))
-		{
-			++page_num;
-			continue;
-		}
-
-		gdk_window_get_origin (GDK_WINDOW (gtk_widget_get_window (tab)),
-				       &x_root, &y_root);
-
-		gtk_widget_get_allocation (tab, &allocation);
-		max_x = x_root + allocation.x + allocation.width;
-		max_y = y_root + allocation.y + allocation.height;
-
-		if (((tab_pos == GTK_POS_TOP) || 
-		     (tab_pos == GTK_POS_BOTTOM)) &&
-		    (abs_x <= max_x))
-		{
-			return page_num;
-		}
-		else if (((tab_pos == GTK_POS_LEFT) || 
-		          (tab_pos == GTK_POS_RIGHT)) && 
-		         (abs_y <= max_y))
-		{
-			return page_num;
-		}
-
-		++page_num;
-	}
-	
-	return AFTER_ALL_TABS;
-}
-
-static gint 
-find_notebook_and_tab_at_pos (gint            abs_x, 
-			      gint            abs_y,
-			      GeditNotebook **notebook,
-			      gint           *page_num)
-{
-	*notebook = find_notebook_at_pointer (abs_x, abs_y);
-	if (*notebook == NULL)
-	{
-		return NOT_IN_APP_WINDOWS;
-	}
-	
-	*page_num = find_tab_num_at_pos (*notebook, abs_x, abs_y);
-
-	if (*page_num < 0)
-	{
-		return *page_num;
-	}
-	else
-	{
-		return 0;
-	}
-}
-
-/**
- * gedit_notebook_move_tab:
- * @src: a #GeditNotebook
- * @dest: a #GeditNotebook
- * @tab: a #GeditTab
- * @dest_position: the position for @tab
- *
- * Moves @tab from @src to @dest.
- * If dest_position is greater than or equal to the number of tabs 
- * of the destination nootebook or negative, tab will be moved to the 
- * end of the tabs.
- */
-void
-gedit_notebook_move_tab (GeditNotebook *src,
-			 GeditNotebook *dest,
-			 GeditTab      *tab,
-			 gint           dest_position)
-{
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (dest));
-	g_return_if_fail (src != dest);
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-
-	/* make sure the tab isn't destroyed while we move it */
-	g_object_ref (tab);
-	gedit_notebook_remove_tab (src, tab);
-	gedit_notebook_add_tab (dest, tab, dest_position, TRUE);
-	g_object_unref (tab);
-}
-
-/**
- * gedit_notebook_reorder_tab:
- * @src: a #GeditNotebook
- * @tab: a #GeditTab
- * @dest_position: the position for @tab
- *
- * Reorders the page containing @tab, so that it appears in @dest_position position.
- * If dest_position is greater than or equal to the number of tabs 
- * of the destination notebook or negative, tab will be moved to the 
- * end of the tabs.
- */
-void
-gedit_notebook_reorder_tab (GeditNotebook *src,
-			    GeditTab      *tab,
-			    gint           dest_position)
-{
-	gint old_position;
-	
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));	
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-
-	old_position = gtk_notebook_page_num (GTK_NOTEBOOK (src), 
-				    	      GTK_WIDGET (tab));
-				    	      
-	if (old_position == dest_position)
-		return;
-
-	gtk_notebook_reorder_child (GTK_NOTEBOOK (src), 
-				    GTK_WIDGET (tab),
-				    dest_position);
-		
-	if (!src->priv->drag_in_progress)
-	{
-		g_signal_emit (G_OBJECT (src), 
-			       signals[TABS_REORDERED], 
-			       0);
-	}
-}
-
-static void
-drag_start (GeditNotebook *notebook,
-	    guint32        time)
-{
-	notebook->priv->drag_in_progress = TRUE;
-
-	/* get a new cursor, if necessary */
-	/* FIXME multi-head */
-	if (cursor == NULL)
-		cursor = gdk_cursor_new (GDK_FLEUR);
-
-	/* grab the pointer */
-	gtk_grab_add (GTK_WIDGET (notebook));
-
-	/* FIXME multi-head */
-	if (!gdk_pointer_is_grabbed ())
-	{
-		gdk_pointer_grab (gtk_widget_get_window (GTK_WIDGET (notebook)),
-				  FALSE,
-				  GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-				  NULL, 
-				  cursor, 
-				  time);
-	}
-}
-
-static void
-drag_stop (GeditNotebook *notebook)
-{
-	if (notebook->priv->drag_in_progress)
-	{
-		g_signal_emit (G_OBJECT (notebook), 
-			       signals[TABS_REORDERED], 
-			       0);
-	}
-
-	notebook->priv->drag_in_progress = FALSE;
-	if (notebook->priv->motion_notify_handler_id != 0)
-	{
-		g_signal_handler_disconnect (G_OBJECT (notebook),
-					     notebook->priv->motion_notify_handler_id);
-		notebook->priv->motion_notify_handler_id = 0;
-	}
-}
-
-/* This function is only called during dnd, we don't need to emit TABS_REORDERED
- * here, instead we do it on drag_stop
- */
-static void
-move_current_tab (GeditNotebook *notebook,
-	          gint           dest_position)
-{
-	gint cur_page_num;
-
-	cur_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-
-	if (dest_position != cur_page_num)
-	{
-		GtkWidget *cur_tab;
-		
-		cur_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-						     cur_page_num);
-						     
-		gedit_notebook_reorder_tab (GEDIT_NOTEBOOK (notebook),
-					    GEDIT_TAB (cur_tab),
-					    dest_position);
-	}
-}
-
-static gboolean
-motion_notify_cb (GeditNotebook  *notebook,
-		  GdkEventMotion *event,
-		  gpointer        data)
-{
-	GeditNotebook *dest;
-	gint page_num;
-	gint result;
-
-	if (notebook->priv->drag_in_progress == FALSE)
-	{
-		if (notebook->priv->tab_drag_and_drop_enabled == FALSE)
-			return FALSE;
-			
-		if (gtk_drag_check_threshold (GTK_WIDGET (notebook),
-					      notebook->priv->x_start,
-					      notebook->priv->y_start,
-					      event->x_root, 
-					      event->y_root))
-		{
-			drag_start (notebook, event->time);
-			return TRUE;
-		}
-
-		return FALSE;
-	}
-
-	result = find_notebook_and_tab_at_pos ((gint)event->x_root,
-					       (gint)event->y_root,
-					       &dest, 
-					       &page_num);
-
-	if (result != NOT_IN_APP_WINDOWS)
-	{
-		if (dest != notebook)
-		{
-			move_current_tab_to_another_notebook (notebook, 
-							      dest,
-						      	      event, 
-						      	      page_num);
-		}
-		else
-		{
-			g_return_val_if_fail (page_num >= -1, FALSE);
-			move_current_tab (notebook, page_num);
-		}
-	}
-
-	return FALSE;
-}
-
-static void
-move_current_tab_to_another_notebook (GeditNotebook  *src,
-				      GeditNotebook  *dest,
-				      GdkEventMotion *event,
-				      gint            dest_position)
-{
-	GeditTab *tab;
-	gint cur_page;
-
-	/* This is getting tricky, the tab was dragged in a notebook
-	 * in another window of the same app, we move the tab
-	 * to that new notebook, and let this notebook handle the
-	 * drag
-	 */
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (dest));
-	g_return_if_fail (dest != src);
-
-	cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (src));
-	tab = GEDIT_TAB (gtk_notebook_get_nth_page (GTK_NOTEBOOK (src), 
-						    cur_page));
-
-	/* stop drag in origin window */
-	/* ungrab the pointer if it's grabbed */
-	drag_stop (src);
-	if (gdk_pointer_is_grabbed ())
-	{
-		gdk_pointer_ungrab (event->time);
-	}
-	gtk_grab_remove (GTK_WIDGET (src));
-
-	gedit_notebook_move_tab (src, dest, tab, dest_position);
-
-	/* start drag handling in dest notebook */
-	dest->priv->motion_notify_handler_id =
-		g_signal_connect (G_OBJECT (dest),
-				  "motion-notify-event",
-				  G_CALLBACK (motion_notify_cb),
-				  NULL);
-
-	drag_start (dest, event->time);
-}
-
-static gboolean
-button_release_cb (GeditNotebook  *notebook,
-		   GdkEventButton *event,
-		   gpointer        data)
-{
-	if (notebook->priv->drag_in_progress)
-	{
-		gint cur_page_num;
-		GtkWidget *cur_page;
-
-		cur_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-						      cur_page_num);
-
-		/* CHECK: I don't follow the code here -- Paolo  */
-		if (!is_in_notebook_window (notebook, event->x_root, event->y_root) &&
-		    (gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) > 1))
-		{
-			/* Tab was detached */
-			g_signal_emit (G_OBJECT (notebook),
-				       signals[TAB_DETACHED], 
-				       0, 
-				       cur_page);
-		}
-
-		/* ungrab the pointer if it's grabbed */
-		if (gdk_pointer_is_grabbed ())
-		{
-			gdk_pointer_ungrab (event->time);
-		}
-		gtk_grab_remove (GTK_WIDGET (notebook));
-	}
-
-	/* This must be called even if a drag isn't happening */
-	drag_stop (notebook);
-
-	return FALSE;
-}
-
-static gboolean
-button_press_cb (GeditNotebook  *notebook,
-		 GdkEventButton *event,
-		 gpointer        data)
-{
-	gint tab_clicked;
-
-	if (notebook->priv->drag_in_progress)
-		return TRUE;
-
-	tab_clicked = find_tab_num_at_pos (notebook,
-					   event->x_root,
-					   event->y_root);
-					   
-	if ((event->button == 1) && 
-	    (event->type == GDK_BUTTON_PRESS) && 
-	    (tab_clicked >= 0))
-	{
-		notebook->priv->x_start = event->x_root;
-		notebook->priv->y_start = event->y_root;
-		
-		notebook->priv->motion_notify_handler_id =
-			g_signal_connect (G_OBJECT (notebook),
-					  "motion-notify-event",
-					  G_CALLBACK (motion_notify_cb), 
-					  NULL);
-	}
-	else if ((event->type == GDK_BUTTON_PRESS) && 
-		 (event->button == 3))
-	{
-		if (tab_clicked == -1)
-		{
-			// CHECK: do we really need it?
-			
-			/* consume event, so that we don't pop up the context menu when
-			 * the mouse if not over a tab label
-			 */
-			return TRUE;
-		}
-		else
-		{
-			/* Switch to the page the mouse is over, but don't consume the event */
-			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 
-						       tab_clicked);
-		}
-	}
-
-	return FALSE;
-}
-
 /**
  * gedit_notebook_new:
  *
@@ -747,32 +451,6 @@ gedit_notebook_new (void)
 	return GTK_WIDGET (g_object_new (GEDIT_TYPE_NOTEBOOK, NULL));
 }
 
-static void
-gedit_notebook_switch_page_cb (GtkNotebook     *notebook,
-                               GtkNotebookPage *page,
-                               guint            page_num,
-                               gpointer         data)
-{
-	GeditNotebook *nb = GEDIT_NOTEBOOK (notebook);
-	GtkWidget *child;
-
-	child = gtk_notebook_get_nth_page (notebook, page_num);
-
-	/* Remove the old page, we dont want to grow unnecessarily
-	 * the list */
-	if (nb->priv->focused_pages)
-	{
-		nb->priv->focused_pages =
-			g_list_remove (nb->priv->focused_pages, child);
-	}
-
-	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
-						 child);
-
-	/* give focus to the tab */
-	gtk_widget_grab_focus (child);
-}
-
 static void
 gedit_notebook_init (GeditNotebook *notebook)
 {
@@ -782,85 +460,19 @@ gedit_notebook_init (GeditNotebook *notebook)
 
 	notebook->priv->show_tabs_mode = GEDIT_NOTEBOOK_SHOW_TABS_ALWAYS;
 	notebook->priv->close_buttons_sensitive = TRUE;
-	notebook->priv->tab_drag_and_drop_enabled = TRUE;
 	
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), TRUE);
+	gtk_notebook_set_group_name (GTK_NOTEBOOK (notebook),
+	                             GEDIT_NOTEBOOK_GROUP_NAME);
+	gtk_container_set_border_width (GTK_CONTAINER (notebook), 0);
 
 	g_settings_bind (notebook->priv->ui_settings,
 			 GEDIT_SETTINGS_SHOW_TABS_MODE,
 			 notebook,
 			 "show-tabs-mode",
 			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
-
-	g_signal_connect (notebook,
-			  "button-press-event",
-			  (GCallback)button_press_cb,
-			  NULL);
-	g_signal_connect (notebook,
-			  "button-release-event",
-			  (GCallback)button_release_cb,
-			  NULL);
-	gtk_widget_add_events (GTK_WIDGET (notebook),
-			       GDK_BUTTON1_MOTION_MASK);
-
-	g_signal_connect_after (G_OBJECT (notebook),
-				"switch_page",
-                                G_CALLBACK (gedit_notebook_switch_page_cb),
-                                NULL);
-}
-
-/*
- * We need to override this because when we don't show the tabs, like in
- * fullscreen we need to have wrap around too
- */
-static gboolean
-gedit_notebook_change_current_page (GtkNotebook *notebook,
-				    gint         offset)
-{
-	gboolean wrap_around;
-	gint current;
-	
-	current = gtk_notebook_get_current_page (notebook);
-
-	if (current != -1)
-	{
-		current = current + offset;
-		
-		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
-			      "gtk-keynav-wrap-around", &wrap_around,
-			      NULL);
-		
-		if (wrap_around)
-		{
-			if (current < 0)
-			{
-				current = gtk_notebook_get_n_pages (notebook) - 1;
-			}
-			else if (current >= gtk_notebook_get_n_pages (notebook))
-			{
-				current = 0;
-			}
-		}
-		
-		gtk_notebook_set_current_page (notebook, current);
-	}
-	else
-	{
-		gtk_widget_error_bell (GTK_WIDGET (notebook));
-	}
-
-	return TRUE;
-}
-
-static void
-close_button_clicked_cb (GeditTabLabel *tab_label, GeditNotebook *notebook)
-{
-	GeditTab *tab;
-
-	tab = gedit_tab_label_get_tab (tab_label);
-	g_signal_emit (notebook, signals[TAB_CLOSE_REQUEST], 0, tab);
 }
 
 static GtkWidget *
@@ -871,42 +483,11 @@ create_tab_label (GeditNotebook *nb,
 
 	tab_label = gedit_tab_label_new (tab);
 
-	g_signal_connect (tab_label,
-			  "close-clicked",
-			  G_CALLBACK (close_button_clicked_cb),
-			  nb);
-
 	g_object_set_data (G_OBJECT (tab), "tab-label", tab_label);
 
 	return tab_label;
 }
 
-static void
-remove_tab_label (GeditNotebook *nb,
-		  GeditTab      *tab)
-{
-	GtkWidget *tab_label;
-
-	tab_label = gedit_tab_label_new (tab);
-
-	g_signal_handlers_disconnect_by_func (tab_label,
-					      G_CALLBACK (close_button_clicked_cb),
-					      nb);
-
-	g_object_set_data (G_OBJECT (tab), "tab-label", NULL);
-}
-
-static GtkWidget *
-get_tab_label (GeditTab *tab)
-{
-	GtkWidget *tab_label;
-
-	tab_label = GTK_WIDGET (g_object_get_data (G_OBJECT (tab), "tab-label"));
-	g_return_val_if_fail (tab_label != NULL, NULL);
-
-	return tab_label;
-}
-
 /**
  * gedit_notebook_add_tab:
  * @nb: a #GeditNotebook
@@ -933,8 +514,12 @@ gedit_notebook_add_tab (GeditNotebook *nb,
 				  GTK_WIDGET (tab),
 				  tab_label,
 				  position);
-
-	update_tabs_visibility (nb, FALSE);
+	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (nb),
+	                                  GTK_WIDGET (tab),
+	                                  TRUE);
+	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (nb),
+	                                 GTK_WIDGET (tab),
+	                                 TRUE);
 
 	/* The signal handler may have reordered the tabs */
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
@@ -951,51 +536,45 @@ gedit_notebook_add_tab (GeditNotebook *nb,
 	}
 }
 
-static void
-smart_tab_switching_on_closure (GeditNotebook *nb,
-				GeditTab      *tab)
-{
-	gboolean jump_to;
-
-	jump_to = GPOINTER_TO_INT (g_object_get_data
-				   (G_OBJECT (tab), "jump_to"));
-
-	if (!jump_to || !nb->priv->focused_pages)
-	{
-		gtk_notebook_next_page (GTK_NOTEBOOK (nb));
-	}
-	else
-	{
-		GList *l;
-		GtkWidget *child;
-		int page_num;
-
-		/* activate the last focused tab */
-		l = g_list_last (nb->priv->focused_pages);
-		child = GTK_WIDGET (l->data);
-		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-						  child);
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), 
-					       page_num);
-	}
-}
-
 static void
 remove_tab (GeditTab      *tab,
-	    GeditNotebook *nb)
+            GeditNotebook *nb)
 {
 	gint position;
-	gint num_pages;
 
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 
-	remove_tab_label (nb, tab);
-	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
+}
 
-	/* If there is no tabs, calling this is pointless */
-	if (--num_pages > 0)
-		update_tabs_visibility (nb, FALSE);
+/**
+ * gedit_notebook_move_tab:
+ * @src: a #GeditNotebook
+ * @dest: a #GeditNotebook
+ * @tab: a #GeditTab
+ * @dest_position: the position for @tab
+ *
+ * Moves @tab from @src to @dest.
+ * If dest_position is greater than or equal to the number of tabs
+ * of the destination nootebook or negative, tab will be moved to the
+ * end of the tabs.
+ */
+void
+gedit_notebook_move_tab (GeditNotebook *src,
+                         GeditNotebook *dest,
+                         GeditTab      *tab,
+                         gint           dest_position)
+{
+	g_return_if_fail (GEDIT_IS_NOTEBOOK (src));
+	g_return_if_fail (GEDIT_IS_NOTEBOOK (dest));
+	g_return_if_fail (src != dest);
+	g_return_if_fail (GEDIT_IS_TAB (tab));
+
+	/* make sure the tab isn't destroyed while we move it */
+	g_object_ref (tab);
+	remove_tab (tab, src);
+	gedit_notebook_add_tab (dest, tab, dest_position, TRUE);
+	g_object_unref (tab);
 }
 
 /**
@@ -1009,23 +588,9 @@ void
 gedit_notebook_remove_tab (GeditNotebook *nb,
 			   GeditTab      *tab)
 {
-	gint position, curr;
-
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 
-	/* Remove the page from the focused pages list */
-	nb->priv->focused_pages =  g_list_remove (nb->priv->focused_pages,
-						  tab);
-
-	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-	curr = gtk_notebook_get_current_page (GTK_NOTEBOOK (nb));
-
-	if (position == curr)
-	{
-		smart_tab_switching_on_closure (nb, tab);
-	}
-
 	remove_tab (tab, nb);
 }
 
@@ -1101,41 +666,34 @@ gedit_notebook_get_close_buttons_sensitive (GeditNotebook *nb)
 	return nb->priv->close_buttons_sensitive;
 }
 
-/**
- * gedit_notebook_set_tab_drag_and_drop_enabled:
- * @nb: a #GeditNotebook
- * @enable: %TRUE to enable the drag and drop
- *
- * Sets whether drag and drop of tabs in the @nb is enabled.
- */
 void
-gedit_notebook_set_tab_drag_and_drop_enabled (GeditNotebook *nb,
-					      gboolean       enable)
+gedit_notebook_collapse_border (GeditNotebook *nb,
+				gboolean       collapse)
 {
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
-	
-	enable = (enable != FALSE);
-	
-	if (enable == nb->priv->tab_drag_and_drop_enabled)
-		return;
-		
-	nb->priv->tab_drag_and_drop_enabled = enable;		
-}
+	GtkRcStyle *rcstyle;
+	gint xthickness;
 
-/**
- * gedit_notebook_get_tab_drag_and_drop_enabled:
- * @nb: a #GeditNotebook
- *
- * Whether the drag and drop is enabled in the @nb.
- *
- * Returns: %TRUE if the drag and drop is enabled.
- */
-gboolean	
-gedit_notebook_get_tab_drag_and_drop_enabled (GeditNotebook *nb)
-{
-	g_return_val_if_fail (GEDIT_IS_NOTEBOOK (nb), TRUE);
-	
-	return nb->priv->tab_drag_and_drop_enabled;
+	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
+
+	nb->priv->collapse_border = collapse;
+
+	if (collapse)
+	{
+		GtkStyle *style;
+
+		style = gtk_widget_get_style (GTK_WIDGET (nb));
+		nb->priv->xthickness = style->xthickness;
+		xthickness = 0;
+	}
+	else
+	{
+		xthickness = nb->priv->xthickness;
+	}
+
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = xthickness;
+	gtk_widget_modify_style (GTK_WIDGET (nb), rcstyle);
+	g_object_unref (rcstyle);
 }
 
 /* ex:ts=8:noet: */
