@@ -92,7 +92,7 @@ gedit_print_preview_get_property (GObject    *object,
 				  GValue     *value,
 				  GParamSpec *pspec)
 {
-	//GeditPrintPreview *preview = GEDIT_PRINT_PREVIEW (object);
+	/* GeditPrintPreview *preview = GEDIT_PRINT_PREVIEW (object); */
 	
 	switch (prop_id)
 	{
@@ -108,7 +108,7 @@ gedit_print_preview_set_property (GObject      *object,
 				  const GValue *value,
 				  GParamSpec   *pspec)
 {
-	//GeditPrintPreview *preview = GEDIT_PRINT_PREVIEW (object);
+	/* GeditPrintPreview *preview = GEDIT_PRINT_PREVIEW (object); */
 	
 	switch (prop_id)
 	{
@@ -121,7 +121,7 @@ gedit_print_preview_set_property (GObject      *object,
 static void
 gedit_print_preview_finalize (GObject *object)
 {
-	//GeditPrintPreview *preview = GEDIT_PRINT_PREVIEW (object);
+	/* GeditPrintPreview *preview = GEDIT_PRINT_PREVIEW (object); */
 
 	G_OBJECT_CLASS (gedit_print_preview_parent_class)->finalize (object);
 }
@@ -151,7 +151,20 @@ gedit_print_preview_class_init (GeditPrintPreviewClass *klass)
 
 	widget_class->grab_focus = gedit_print_preview_grab_focus;
 
-	g_type_class_add_private (object_class, sizeof(GeditPrintPreviewPrivate));	
+	g_type_class_add_private (object_class, sizeof (GeditPrintPreviewPrivate));	
+}
+
+static void
+get_adjustments (GeditPrintPreview  *preview,
+		 GtkAdjustment     **hadj,
+		 GtkAdjustment     **vadj)
+{
+	GeditPrintPreviewPrivate *priv;
+
+	priv = preview->priv;
+
+	*hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (priv->layout));
+	*vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (priv->layout));
 }
 
 static void
@@ -248,19 +261,18 @@ set_zoom_factor (GeditPrintPreview *preview,
 static void
 set_zoom_fit_to_size (GeditPrintPreview *preview)
 {
-	GeditPrintPreviewPrivate *priv;	
+	GeditPrintPreviewPrivate *priv;
+	GtkAdjustment *hadj, *vadj;
 	double width, height;
 	double p_width, p_height;
 	double zoomx, zoomy;
 
 	priv = preview->priv;
 
-	g_object_get (gtk_layout_get_hadjustment (GTK_LAYOUT (priv->layout)),
-		      "page-size", &width,
-		      NULL);
-	g_object_get (gtk_layout_get_vadjustment (GTK_LAYOUT (priv->layout)),
-		      "page-size", &height,
-		      NULL);
+	get_adjustments (preview, &hadj, &vadj);
+
+	g_object_get (hadj, "page-size", &width, NULL);
+	g_object_get (vadj, "page-size", &height, NULL);
 
 	width /= priv->cols;
 	height /= priv->rows;
@@ -314,7 +326,8 @@ zoom_out (GeditPrintPreview *preview)
 }
 
 static void
-goto_page (GeditPrintPreview *preview, gint page)
+goto_page (GeditPrintPreview *preview,
+	   gint               page)
 {
 	gchar c[32];
 
@@ -443,31 +456,35 @@ page_entry_focus_out (GtkWidget         *widget,
 }
 
 static void
-on_1x1_clicked (GtkMenuItem *i, GeditPrintPreview *preview)
+on_1x1_clicked (GtkMenuItem       *i,
+		GeditPrintPreview *preview)
 {
 	set_rows_and_cols (preview, 1, 1);
 }
 
 static void
-on_1x2_clicked (GtkMenuItem *i, GeditPrintPreview *preview)
+on_1x2_clicked (GtkMenuItem       *i,
+		GeditPrintPreview *preview)
 {
 	set_rows_and_cols (preview, 1, 2);
 }
 
 static void
-on_2x1_clicked (GtkMenuItem *i, GeditPrintPreview *preview)
+on_2x1_clicked (GtkMenuItem       *i,
+		GeditPrintPreview *preview)
 {
 	set_rows_and_cols (preview, 2, 1);
 }
 
 static void
-on_2x2_clicked (GtkMenuItem *i, GeditPrintPreview *preview)
+on_2x2_clicked (GtkMenuItem       *i,
+		GeditPrintPreview *preview)
 {
 	set_rows_and_cols (preview, 2, 2);
 }
 
 static void
-multi_button_clicked (GtkWidget	 *button,
+multi_button_clicked (GtkWidget	        *button,
 		      GeditPrintPreview *preview)
 {
 	GtkWidget *m, *i;
@@ -726,8 +743,7 @@ get_page_at_coords (GeditPrintPreview *preview,
 	if (priv->tile_h <= 0 || priv->tile_h <= 0)
 		return -1;
 
-	hadj = gtk_layout_get_hadjustment (GTK_LAYOUT (priv->layout));
-	vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (priv->layout));
+	get_adjustments (preview, &hadj, &vadj);
 
 	x += gtk_adjustment_get_value (hadj);
 	y += gtk_adjustment_get_value (vadj);
@@ -788,8 +804,7 @@ preview_layout_key_press (GtkWidget         *widget,
 
 	priv = preview->priv;
 
-	hadj = gtk_layout_get_hadjustment (GTK_LAYOUT (priv->layout));
-	vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (priv->layout));
+	get_adjustments (preview, &hadj, &vadj);
 
 	x = gtk_adjustment_get_value (hadj);
 	y = gtk_adjustment_get_value (vadj);
@@ -810,118 +825,119 @@ preview_layout_key_press (GtkWidget         *widget,
 	hstep = 10;
 	vstep = 10;
 
-	switch (event->keyval) {
-	case '1':
-		set_zoom_fit_to_size (preview);
-		break;
-	case '+':
-	case '=':
-	case GDK_KP_Add:
-		zoom_in (preview);
-		break;
-	case '-':
-	case '_':
-	case GDK_KP_Subtract:
-		zoom_out (preview);
-		break;
-	case GDK_KP_Right:
-	case GDK_Right:
-		if (event->state & GDK_SHIFT_MASK)
-			x = hupper - hpage;
-		else
-			x = MIN (hupper - hpage, x + hstep);
-		domove = TRUE;
-		break;
-	case GDK_KP_Left:
-	case GDK_Left:
-		if (event->state & GDK_SHIFT_MASK)
-			x = hlower;
-		else
-			x = MAX (hlower, x - hstep);
-		domove = TRUE;
-		break;
-	case GDK_KP_Up:
-	case GDK_Up:
-		if (event->state & GDK_SHIFT_MASK)
-			goto page_up;
-		y = MAX (vlower, y - vstep);
-		domove = TRUE;
-		break;
-	case GDK_KP_Down:
-	case GDK_Down:
-		if (event->state & GDK_SHIFT_MASK)
-			goto page_down;
-		y = MIN (vupper - vpage, y + vstep);
-		domove = TRUE;
-		break;
-	case GDK_KP_Page_Up:
-	case GDK_Page_Up:
-	case GDK_Delete:
-	case GDK_KP_Delete:
-	case GDK_BackSpace:
-	page_up:
-		if (y <= vlower)
-		{
-			if (preview->priv->cur_page > 0)
+	switch (event->keyval)
+	{
+		case '1':
+			set_zoom_fit_to_size (preview);
+			break;
+		case '+':
+		case '=':
+		case GDK_KEY_KP_Add:
+			zoom_in (preview);
+			break;
+		case '-':
+		case '_':
+		case GDK_KEY_KP_Subtract:
+			zoom_out (preview);
+			break;
+		case GDK_KEY_KP_Right:
+		case GDK_KEY_Right:
+			if (event->state & GDK_SHIFT_MASK)
+				x = hupper - hpage;
+			else
+				x = MIN (hupper - hpage, x + hstep);
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_Left:
+		case GDK_KEY_Left:
+			if (event->state & GDK_SHIFT_MASK)
+				x = hlower;
+			else
+				x = MAX (hlower, x - hstep);
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_Up:
+		case GDK_KEY_Up:
+			if (event->state & GDK_SHIFT_MASK)
+				goto page_up;
+			y = MAX (vlower, y - vstep);
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_Down:
+		case GDK_KEY_Down:
+			if (event->state & GDK_SHIFT_MASK)
+				goto page_down;
+			y = MIN (vupper - vpage, y + vstep);
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_Page_Up:
+		case GDK_KEY_Page_Up:
+		case GDK_KEY_Delete:
+		case GDK_KEY_KP_Delete:
+		case GDK_KEY_BackSpace:
+		page_up:
+			if (y <= vlower)
 			{
-				goto_page (preview, preview->priv->cur_page - 1);
-				y = (vupper - vpage);
+				if (preview->priv->cur_page > 0)
+				{
+					goto_page (preview, preview->priv->cur_page - 1);
+					y = (vupper - vpage);
+				}
 			}
-		}
-		else
-		{
-			y = vlower;
-		}
-		domove = TRUE;
-		break;
-	case GDK_KP_Page_Down:
-	case GDK_Page_Down:
-	case ' ':
-	page_down:
-		if (y >= (vupper - vpage))
-		{
-			if (preview->priv->cur_page < preview->priv->n_pages - 1)
+			else
 			{
-				goto_page (preview, preview->priv->cur_page + 1);
 				y = vlower;
 			}
-		}
-		else
-		{
-			y = (vupper - vpage);
-		}
-		domove = TRUE;
-		break;
-	case GDK_KP_Home:
-	case GDK_Home:
-		goto_page (preview, 0);
-		y = 0;
-		domove = TRUE;
-		break;
-	case GDK_KP_End:
-	case GDK_End:
-		goto_page (preview, preview->priv->n_pages - 1);
-		y = 0;
-		domove = TRUE;
-		break;
-	case GDK_Escape:
-		gtk_widget_destroy (GTK_WIDGET (preview));
-		break;
-	case 'c':
-		if (event->state & GDK_MOD1_MASK)
-		{
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_Page_Down:
+		case GDK_KEY_Page_Down:
+		case ' ':
+		page_down:
+			if (y >= (vupper - vpage))
+			{
+				if (preview->priv->cur_page < preview->priv->n_pages - 1)
+				{
+					goto_page (preview, preview->priv->cur_page + 1);
+					y = vlower;
+				}
+			}
+			else
+			{
+				y = (vupper - vpage);
+			}
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_Home:
+		case GDK_KEY_Home:
+			goto_page (preview, 0);
+			y = 0;
+			domove = TRUE;
+			break;
+		case GDK_KEY_KP_End:
+		case GDK_KEY_End:
+			goto_page (preview, preview->priv->n_pages - 1);
+			y = 0;
+			domove = TRUE;
+			break;
+		case GDK_KEY_Escape:
 			gtk_widget_destroy (GTK_WIDGET (preview));
-		}
-		break;
-	case 'p':
-		if (event->state & GDK_MOD1_MASK)
-		{
-			gtk_widget_grab_focus (preview->priv->page_entry);
-		}
-		break;
-	default:
-		/* by default do not stop the default handler */
-		ret = FALSE;
+			break;
+		case 'c':
+			if (event->state & GDK_MOD1_MASK)
+			{
+				gtk_widget_destroy (GTK_WIDGET (preview));
+			}
+			break;
+		case 'p':
+			if (event->state & GDK_MOD1_MASK)
+			{
+				gtk_widget_grab_focus (preview->priv->page_entry);
+			}
+			break;
+		default:
+			/* by default do not stop the default handler */
+			ret = FALSE;
 	}
 
 	if (domove)
@@ -945,7 +961,7 @@ create_preview_layout (GeditPrintPreview *preview)
 	priv = preview->priv;
 
 	priv->layout = gtk_layout_new (NULL, NULL);
-//	gtk_widget_set_double_buffered (priv->layout, FALSE);
+	/* gtk_widget_set_double_buffered (priv->layout, FALSE); */
 
 	atko = gtk_widget_get_accessible (GTK_WIDGET (priv->layout));
 	atk_object_set_name (atko, _("Page Preview"));
@@ -956,7 +972,7 @@ create_preview_layout (GeditPrintPreview *preview)
 			       GDK_BUTTON_PRESS_MASK |
 			       GDK_KEY_PRESS_MASK);
 
-	GTK_WIDGET_SET_FLAGS (priv->layout, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (priv->layout, TRUE);
 
   	g_signal_connect (priv->layout,
 			  "key-press-event",
@@ -1001,7 +1017,7 @@ gedit_print_preview_init (GeditPrintPreview *preview)
 	create_bar (preview);
 	create_preview_layout (preview);
 
-	// FIXME
+	/* FIXME */
 	priv->cur_page = 0;
 	priv->paper_w = 0;
 	priv->paper_h = 0;
@@ -1104,53 +1120,53 @@ draw_page (cairo_t           *cr,
 }
 
 static gboolean
-preview_expose (GtkWidget         *widget,
-		GdkEventExpose    *event,
-		GeditPrintPreview *preview)
+preview_draw (GtkWidget         *widget,
+	      cairo_t           *cr,
+	      GeditPrintPreview *preview)
 {
 	GeditPrintPreviewPrivate *priv;
 	GdkWindow *bin_window;
-	cairo_t *cr;
 	gint pg;
 	gint i, j;
 
 	priv = preview->priv;
 
 	bin_window = gtk_layout_get_bin_window (GTK_LAYOUT (priv->layout));
-	if (event->window != bin_window)
-		return FALSE;
 
-	cr = gdk_cairo_create (bin_window);
-
-	gdk_cairo_rectangle (cr, &event->area);
-	cairo_clip (cr);
-
-	/* get the first page to display */
-	pg = get_first_page_displayed (preview);
-
-	for (i = 0; i < priv->cols; ++i)
+	if (gtk_cairo_should_draw_window (cr, bin_window))
 	{
-		for (j = 0; j < priv->rows; ++j)
+		cairo_save (cr);
+
+		gtk_cairo_transform_to_window (cr, widget, bin_window);
+
+		/* get the first page to display */
+		pg = get_first_page_displayed (preview);
+
+		for (i = 0; i < priv->cols; ++i)
 		{
-			if (!gtk_print_operation_preview_is_selected (priv->gtk_preview,
-								      pg))
+			for (j = 0; j < priv->rows; ++j)
 			{
-				continue;
+				if (!gtk_print_operation_preview_is_selected (priv->gtk_preview,
+									      pg))
+				{
+					continue;
+				}
+
+				if (pg == priv->n_pages)
+					break;
+
+				draw_page (cr,
+					   j * priv->tile_w,
+					   i * priv->tile_h,
+					   pg,
+					   preview);
+
+				++pg;
 			}
-
-			if (pg == priv->n_pages)
-				break;
-
-			draw_page (cr,
-				   j * priv->tile_w,
-				   i * priv->tile_h,
-				   pg,
-				   preview);
-
-			++pg;
 		}
+
+		cairo_restore (cr);
 	}
-	cairo_destroy (cr);
 
 	return TRUE;
 }
@@ -1181,7 +1197,7 @@ set_n_pages (GeditPrintPreview *preview,
 
 	preview->priv->n_pages = n_pages;
 
-	// FIXME: count the visible pages
+	/* FIXME: count the visible pages */
 
 	str =  g_strdup_printf ("%d", n_pages);
 	gtk_label_set_markup (GTK_LABEL (preview->priv->last), str);
@@ -1206,8 +1222,8 @@ preview_ready (GtkPrintOperationPreview *gtk_preview,
 
 	/* let the default gtklayout handler clear the background */
 	g_signal_connect_after (preview->priv->layout,
-				"expose-event",
-				G_CALLBACK (preview_expose),
+				"draw",
+				G_CALLBACK (preview_draw),
 				preview);
 
 	gtk_widget_queue_draw (preview->priv->layout);
@@ -1325,3 +1341,4 @@ gedit_print_preview_new (GtkPrintOperation        *op,
 	return GTK_WIDGET (preview);
 }
 
+/* ex:ts=8:noet: */
